from ._vpc import _vpc
from .errors import ErrorHandler

class PeeringConnection(_vpc):
    def __init__(self, region: str, peering_connection_id: str = None, aws_access_key: str = None, aws_access_secret_key: str = None):
        self._peering_connection_id = peering_connection_id
        self._region = region
        self._aws_access_key = aws_access_key
        self._aws_access_secret_key = aws_access_secret_key
        self._error_handler = ErrorHandler()

        super().__init__(region=region, aws_access_key=self._aws_access_key, aws_access_secret_key=self._aws_access_secret_key)
        del self._aws_access_key, self._aws_access_secret_key

    def create_peering_connection(self, vpc_id: str, peer_vpc_id: str, peer_region: str = None, peer_owner_id: str = None, peering_name: str = "constellation-peering-connection") -> dict:
        result = super()._create_vpc_peering_connection(vpc_id, peer_vpc_id, peer_region, peer_owner_id, peering_name)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        self._peering_connection_id = result.get('VpcPeeringConnectionId')
        return result

    def accept_peering_connection(self) -> dict:
        if not self._peering_connection_id:
            return {"Error": "VPC Peering Connection ID is required to accept the connection"}

        result = super()._accept_vpc_peering_connection(self._peering_connection_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def delete_peering_connection(self) -> dict:
        if not self._peering_connection_id:
            return {"Error": "VPC Peering Connection ID is required to delete the connection"}

        result = super()._delete_vpc_peering_connection(self._peering_connection_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def describe_peering_connections(self, vpc_id: str = None, peer_vpc_id: str = None) -> dict:
        vpc_id = vpc_id or self._vpc_id  # Use self._vpc_id if vpc_id is None
        peer_vpc_id = peer_vpc_id or self._peer_vpc_id  # Use self._peer_vpc_id if peer_vpc_id is None

        result = super()._describe_vpc_peering_connections(peering_connection_id=self._peering_connection_id, vpc_id=vpc_id, peer_vpc_id=peer_vpc_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def reject_peering_connection(self) -> dict:
        if not self._peering_connection_id:
            return {"Error": "VPC Peering Connection ID is required to reject the connection"}

        result = super()._reject_vpc_peering_connection(self._peering_connection_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def describe_peering_connection_requests(self, vpc_id: str = None) -> dict:
        vpc_id = vpc_id or self._vpc_id  # Use self._vpc_id if vpc_id is None
        result = super()._describe_vpc_peering_connection_requests(vpc_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    @property
    def peering_connection_id(self):
        return self._peering_connection_id



__name__ = "constellation_vpc"
__version__ = "0.5.0a1"
__author__ = "Coulter Stutz"
__email__ = "coulterstutz@gmail.com"
__license__ = "MIT"
from .subnet import Subnet
from .vpc import VPC
from .routing_table import RoutingTable
from .internet_gateway import InternetGateway
from .nat_gateway import NatGateway
from .peering_connection import PeeringConnection
__all__ = ["Subnet", "VPC", "RoutingTable", "InternetGateway", "NatGateway", "PeeringConnection"]


from ._vpc import _vpc
from .errors import ErrorHandler

class NatGateway(_vpc):
    def __init__(self, region: str, nat_gateway_id: str = None, aws_access_key: str = None, aws_access_secret_key: str = None):
        self._nat_gateway_id = nat_gateway_id
        self._region = region
        self._aws_access_key = aws_access_key
        self._aws_access_secret_key = aws_access_secret_key
        self._error_handler = ErrorHandler()

        super().__init__(region=region, aws_access_key=self._aws_access_key, aws_access_secret_key=self._aws_access_secret_key)
        del self._aws_access_key, self._aws_access_secret_key

    def create_nat_gateway(self, subnet_id: str, allocation_id: str, nat_gateway_name: str = "constellation-nat-gateway") -> dict:
        nat_creation_result = super()._create_nat_gateway(subnet_id, allocation_id, nat_gateway_name)

        if "Error" in nat_creation_result:
            self._error_handler.parse_and_raise(nat_creation_result)

        self._nat_gateway_id = nat_creation_result.get('NatGatewayId')
        return nat_creation_result

    def delete_nat_gateway(self) -> dict:
        if not self._nat_gateway_id:
            return {"Error": "NAT Gateway ID is required to delete"}

        delete_result = super()._delete_nat_gateway(self._nat_gateway_id)

        if "Error" in delete_result:
            self._error_handler.parse_and_raise(delete_result)

        return delete_result

    def describe_nat_gateways(self, vpc_id: str = None, subnet_id: str = None) -> dict:
        return super()._describe_nat_gateways(vpc_id, subnet_id)

    def associate_nat_gateway(self, route_table_id: str) -> dict:
        if not self._nat_gateway_id:
            return {"Error": "NAT Gateway ID is required to associate with route table"}

        associate_result = super()._associate_nat_gateway(self._nat_gateway_id, route_table_id)

        if "Error" in associate_result:
            self._error_handler.parse_and_raise(associate_result)

        return associate_result

    def disassociate_nat_gateway(self, route_table_id: str) -> dict:
        disassociate_result = super()._disassociate_nat_gateway(route_table_id)

        if "Error" in disassociate_result:
            self._error_handler.parse_and_raise(disassociate_result)

        return disassociate_result

    @property
    def nat_gateway_id(self):
        return self._nat_gateway_id




from ._vpc import _vpc
from .errors import ErrorHandler

class InternetGateway(_vpc):
    def __init__(self, region: str, igw_id: str = None, aws_access_key: str = None, aws_access_secret_key: str = None):
        self._igw_id = igw_id
        self._region = region
        self._aws_access_key = aws_access_key
        self._aws_access_secret_key = aws_access_secret_key
        self._error_handler = ErrorHandler()

        super().__init__(region=region, aws_access_key=self._aws_access_key, aws_access_secret_key=self._aws_access_secret_key)
        del self._aws_access_key, self._aws_access_secret_key

    def create_internet_gateway(self, igw_name: str = "constellation-igw") -> dict:
        igw_creation_result = super()._create_internet_gateway(igw_name)

        if "Error" in igw_creation_result:
            return igw_creation_result

        return igw_creation_result

    def attach_internet_gateway(self, vpc_id: str) -> dict:
        if not self._igw_id:
            return {"Error": "Internet Gateway ID is required to attach to VPC"}

        attach_result = super()._attach_internet_gateway(self._igw_id, vpc_id)

        if "Error" in attach_result:
            return attach_result

        return attach_result

    def detach_internet_gateway(self, vpc_id: str) -> dict:
        if not self._igw_id:
            return {"Error": "Internet Gateway ID is required to detach from VPC"}

        detach_result = super()._detach_internet_gateway(self._igw_id, vpc_id)

        if "Error" in detach_result:
            return detach_result

        return detach_result

    def delete_internet_gateway(self) -> dict:
        if not self._igw_id:
            return {"Error": "Internet Gateway ID is required to delete"}

        delete_result = super()._delete_internet_gateway(self._igw_id)

        if "Error" in delete_result:
            return delete_result

        return delete_result

    def describe_internet_gateways(self, vpc_id: str = None) -> dict:
        return super()._describe_internet_gateways(vpc_id)




from .errors import ClientNotFoundError
import subprocess as _subprocess
import json

class _vpc:
    def __init__(self, region: str = None, aws_access_key: str = None, aws_access_secret_key: str = None):
        self.region_name = region
        self._access_key = aws_access_key
        self._secret_key = aws_access_secret_key

    def _run_aws_command(self, cmd: list) -> dict:
        env = None
        if self._access_key and self._secret_key:
            env = {
                "AWS_ACCESS_KEY_ID": self._access_key,
                "AWS_SECRET_ACCESS_KEY": self._secret_key
            }
        try:
            process = _subprocess.Popen(cmd, stdout=_subprocess.PIPE, stderr=_subprocess.PIPE, text=True, env=env)
            stdout, stderr = process.communicate()
        except FileNotFoundError:
            raise ClientNotFoundError()
        if process.returncode != 0:
            return {"Error": stderr}

        try:
            return json.loads(stdout)
        except json.JSONDecodeError:
            return {"Error": "Failed to parse JSON output"}

    def _describe_route_table(self, route_table_id: str) -> dict:
        cmd = [
            "aws", "ec2", "describe-route-tables",
            "--route-table-ids", route_table_id,
            "--region", self.region_name
        ]
        return self._run_aws_command(cmd)

    def _create_route_table(self, vpc_id: str, route_table_name: str = "constellation-route-table") -> dict:
        cmd = [
            "aws", "ec2", "create-route-table",
            "--vpc-id", vpc_id,
            "--region", self.region_name
        ]

        route_table_creation_result = self._run_aws_command(cmd)

        if "Error" in route_table_creation_result:
            return route_table_creation_result

        route_table_id = route_table_creation_result.get('RouteTable', {}).get('RouteTableId')
        if not route_table_id:
            return {"Error": "Route Table ID not found in creation response"}

        tag_result = self._run_aws_command([
            "aws", "ec2", "create-tags",
            "--resources", route_table_id,
            "--tags", f"Key=Name,Value={route_table_name}",
            "--region", self.region_name
        ])

        if "Error" in tag_result:
            return tag_result

        return {"RouteTableId": route_table_id, "RouteTableName": route_table_name, "TagResult": tag_result}

    def _delete_route_table(self, route_table_id: str) -> dict:
        cmd = [
            "aws", "ec2", "delete-route-table",
            "--route-table-id", route_table_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _associate_route_table(self, route_table_id: str, subnet_id: str) -> dict:
        cmd = [
            "aws", "ec2", "associate-route-table",
            "--route-table-id", route_table_id,
            "--subnet-id", subnet_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _disassociate_route_table(self, association_id: str) -> dict:
        cmd = [
            "aws", "ec2", "disassociate-route-table",
            "--association-id", association_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _replace_route_table_association(self, association_id: str, route_table_id: str) -> dict:
        cmd = [
            "aws", "ec2", "replace-route-table-association",
            "--association-id", association_id,
            "--route-table-id", route_table_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _create_route(self, route_table_id: str, destination_cidr_block: str, gateway_id: str = None, nat_gateway_id: str = None) -> dict:
        cmd = [
            "aws", "ec2", "create-route",
            "--route-table-id", route_table_id,
            "--destination-cidr-block", destination_cidr_block,
            "--region", self.region_name
        ]

        if gateway_id:
            cmd.extend(["--gateway-id", gateway_id])

        if nat_gateway_id:
            cmd.extend(["--nat-gateway-id", nat_gateway_id])

        return self._run_aws_command(cmd)

    def _delete_route(self, route_table_id: str, destination_cidr_block: str) -> dict:
        cmd = [
            "aws", "ec2", "delete-route",
            "--route-table-id", route_table_id,
            "--destination-cidr-block", destination_cidr_block,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _describe_route_tables(self, vpc_id: str) -> dict:
        cmd = [
            "aws", "ec2", "describe-route-tables",
            "--filters", f"Name=vpc-id,Values={vpc_id}",
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _describe_subnet(self, subnet_id: str) -> dict:
        cmd = [
            "aws", "ec2", "describe-subnets",
            "--subnet-ids", subnet_id,
            "--region", self.region_name
        ]
        return self._run_aws_command(cmd)

    def _describe_subnets(self, vpc_id: str) -> dict:
        cmd = [
            "aws", "ec2", "describe-subnets",
            "--filters", f"Name=vpc-id,Values={vpc_id}",
            "--region", self.region_name
        ]
        return self._run_aws_command(cmd)

    def _create_subnet(self, vpc_id: str, cidr_block: str, availability_zone: str = None, subnet_name: str = "constellation-subnet") -> dict:
        cmd = [
            "aws", "ec2", "create-subnet",
            "--vpc-id", vpc_id,
            "--cidr-block", cidr_block,
            "--region", self.region_name
        ]

        if availability_zone:
            cmd.extend(["--availability-zone", availability_zone])

        subnet_creation_result = self._run_aws_command(cmd)

        if "Error" in subnet_creation_result:
            return subnet_creation_result

        subnet_id = subnet_creation_result.get('Subnet', {}).get('SubnetId')
        if not subnet_id:
            return {"Error": "Subnet ID not found in creation response"}

        tag_result = self._run_aws_command([
            "aws", "ec2", "create-tags",
            "--resources", subnet_id,
            "--tags", f"Key=Name,Value={subnet_name}",
            "--region", self.region_name
        ])

        if "Error" in tag_result:
            return tag_result

        return {"SubnetId": subnet_id, "SubnetName": subnet_name, "TagResult": tag_result}

    def _delete_subnet(self, subnet_id: str) -> dict:
        cmd = [
            "aws", "ec2", "delete-subnet",
            "--subnet-id", subnet_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _associate_subnet_cidr_block(self, subnet_id: str, cidr_block: str) -> dict:
        cmd = [
            "aws", "ec2", "associate-subnet-cidr-block",
            "--subnet-id", subnet_id,
            "--cidr-block", cidr_block,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _disassociate_subnet_cidr_block(self, association_id: str) -> dict:
        cmd = [
            "aws", "ec2", "disassociate-subnet-cidr-block",
            "--association-id", association_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _get_subnet_cidr_reservations(self, subnet_id: str) -> dict:
        cmd = [
            "aws", "ec2", "get-subnet-cidr-reservations",
            "--subnet-id", subnet_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _modify_subnet_attribute(self, subnet_id: str, attribute_name: str, attribute_value) -> dict:
        cmd = [
            "aws", "ec2", "modify-subnet-attribute",
            "--subnet-id", subnet_id,
            f"--{attribute_name}", str(attribute_value).lower(),
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _create_default_vpc(self, subnet_name: str = "constellation-subnet") -> dict:
        cmd = [
            "aws", "ec2", "create-default-vpc",
            "--region", self.region_name
        ]
        vpc_creation_result = self._run_aws_command(cmd)

        if "Error" in vpc_creation_result:
            return vpc_creation_result

        vpc_id = vpc_creation_result.get('Vpc', {}).get('VpcId')
        if not vpc_id:
            return {"Error": "VPC ID not found in creation response"}

        subnets = self._run_aws_command([
            "aws", "ec2", "describe-subnets",
            "--filters", f"Name=vpc-id,Values={vpc_id}",
            "--region", self.region_name
        ])
        if "Error" in subnets:
            return subnets

        subnet_id = subnets.get('Subnets', [])[0].get('SubnetId')
        if not subnet_id:
            return {"Error": "Subnet ID not found"}

        tag_result = self._run_aws_command([
            "aws", "ec2", "create-tags",
            "--resources", subnet_id,
            "--tags", f"Key=Name,Value={subnet_name}",
            "--region", self.region_name
        ])

        if "Error" in tag_result:
            return tag_result

        return {"VpcId": vpc_id, "SubnetId": subnet_id, "SubnetName": subnet_name, "TagResult": tag_result}

    def _describe_vpc(self, vpc_id: str) -> dict:
        cmd = [
            "aws", "ec2", "describe-vpcs",
            "--vpc-ids", vpc_id,
            "--region", self.region_name
        ]
        return self._run_aws_command(cmd)

    def _create_vpc(self, cidr_block: str, vpc_name: str = "constellation-vpc") -> dict:
        cmd = [
            "aws", "ec2", "create-vpc",
            "--cidr-block", cidr_block,
            "--region", self.region_name
        ]

        vpc_creation_result = self._run_aws_command(cmd)

        if "Error" in vpc_creation_result:
            return vpc_creation_result

        vpc_id = vpc_creation_result.get('Vpc', {}).get('VpcId')
        if not vpc_id:
            return {"Error": "VPC ID not found in creation response"}

        tag_result = self._run_aws_command([
            "aws", "ec2", "create-tags",
            "--resources", vpc_id,
            "--tags", f"Key=Name,Value={vpc_name}",
            "--region", self.region_name
        ])

        if "Error" in tag_result:
            return tag_result

        return {"VpcId": vpc_id, "VpcName": vpc_name, "TagResult": tag_result}

    def _delete_vpc(self, vpc_id: str) -> dict:
        cmd = [
            "aws", "ec2", "delete-vpc",
            "--vpc-id", vpc_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _modify_vpc_attribute(self, vpc_id: str, attribute_name: str, attribute_value) -> dict:
        cmd = [
            "aws", "ec2", "modify-vpc-attribute",
            "--vpc-id", vpc_id,
            f"--{attribute_name}", str(attribute_value).lower(),
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _associate_vpc_cidr_block(self, vpc_id: str, cidr_block: str) -> dict:
        cmd = [
            "aws", "ec2", "associate-vpc-cidr-block",
            "--vpc-id", vpc_id,
            "--cidr-block", cidr_block,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _disassociate_vpc_cidr_block(self, association_id: str) -> dict:
        cmd = [
            "aws", "ec2", "disassociate-vpc-cidr-block",
            "--association-id", association_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _describe_vpc_cidr_reservations(self, vpc_id: str) -> dict:
        cmd = [
            "aws", "ec2", "describe-vpc-cidr-block-associations",
            "--vpc-id", vpc_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _create_internet_gateway(self, igw_name: str = "constellation-igw") -> dict:
        cmd = [
            "aws", "ec2", "create-internet-gateway",
            "--region", self.region_name
        ]

        igw_creation_result = self._run_aws_command(cmd)

        if "Error" in igw_creation_result:
            return igw_creation_result

        igw_id = igw_creation_result.get('InternetGateway', {}).get('InternetGatewayId')
        if not igw_id:
            return {"Error": "Internet Gateway ID not found in creation response"}

        tag_result = self._run_aws_command([
            "aws", "ec2", "create-tags",
            "--resources", igw_id,
            "--tags", f"Key=Name,Value={igw_name}",
            "--region", self.region_name
        ])

        if "Error" in tag_result:
            return tag_result

        return {"InternetGatewayId": igw_id, "InternetGatewayName": igw_name, "TagResult": tag_result}

    def _attach_internet_gateway(self, igw_id: str, vpc_id: str) -> dict:
        cmd = [
            "aws", "ec2", "attach-internet-gateway",
            "--internet-gateway-id", igw_id,
            "--vpc-id", vpc_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _detach_internet_gateway(self, igw_id: str, vpc_id: str) -> dict:
        cmd = [
            "aws", "ec2", "detach-internet-gateway",
            "--internet-gateway-id", igw_id,
            "--vpc-id", vpc_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _delete_internet_gateway(self, igw_id: str) -> dict:
        cmd = [
            "aws", "ec2", "delete-internet-gateway",
            "--internet-gateway-id", igw_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _describe_internet_gateways(self, vpc_id: str = None) -> dict:
        cmd = [
            "aws", "ec2", "describe-internet-gateways",
            "--region", self.region_name
        ]

        if vpc_id:
            cmd.extend(["--filters", f"Name=attachment.vpc-id,Values={vpc_id}"])

        return self._run_aws_command(cmd)

    def _create_nat_gateway(self, subnet_id: str, allocation_id: str, nat_gateway_name: str = "constellation-nat-gateway") -> dict:
        cmd = [
            "aws", "ec2", "create-nat-gateway",
            "--subnet-id", subnet_id,
            "--allocation-id", allocation_id,
            "--region", self.region_name
        ]

        nat_creation_result = self._run_aws_command(cmd)

        if "Error" in nat_creation_result:
            return nat_creation_result

        nat_gateway_id = nat_creation_result.get('NatGateway', {}).get('NatGatewayId')
        if not nat_gateway_id:
            return {"Error": "NAT Gateway ID not found in creation response"}

        tag_result = self._run_aws_command([
            "aws", "ec2", "create-tags",
            "--resources", nat_gateway_id,
            "--tags", f"Key=Name,Value={nat_gateway_name}",
            "--region", self.region_name
        ])

        if "Error" in tag_result:
            return tag_result

        return {"NatGatewayId": nat_gateway_id, "NatGatewayName": nat_gateway_name, "TagResult": tag_result}

    def _delete_nat_gateway(self, nat_gateway_id: str) -> dict:
        cmd = [
            "aws", "ec2", "delete-nat-gateway",
            "--nat-gateway-id", nat_gateway_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _describe_nat_gateways(self, vpc_id: str = None, subnet_id: str = None) -> dict:
        cmd = [
            "aws", "ec2", "describe-nat-gateways",
            "--region", self.region_name
        ]

        if vpc_id:
            cmd.extend(["--filter", f"Name=vpc-id,Values={vpc_id}"])
        if subnet_id:
            cmd.extend(["--filter", f"Name=subnet-id,Values={subnet_id}"])

        return self._run_aws_command(cmd)

    def _associate_nat_gateway(self, nat_gateway_id: str, route_table_id: str) -> dict:
        cmd = [
            "aws", "ec2", "create-route",
            "--route-table-id", route_table_id,
            "--nat-gateway-id", nat_gateway_id,
            "--destination-cidr-block", "0.0.0.0/0",
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _disassociate_nat_gateway(self, route_table_id: str) -> dict:
        cmd = [
            "aws", "ec2", "delete-route",
            "--route-table-id", route_table_id,
            "--destination-cidr-block", "0.0.0.0/0",
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _create_vpc_peering_connection(self, vpc_id: str, peer_vpc_id: str, peer_region: str = None,
                                       peer_owner_id: str = None,
                                       peering_name: str = "constellation-peering-connection") -> dict:
        cmd = [
            "aws", "ec2", "create-vpc-peering-connection",
            "--vpc-id", vpc_id,
            "--peer-vpc-id", peer_vpc_id,
            "--region", self.region_name
        ]

        if peer_region:
            cmd.extend(["--peer-region", peer_region])

        if peer_owner_id:
            cmd.extend(["--peer-owner-id", peer_owner_id])

        peering_creation_result = self._run_aws_command(cmd)

        if "Error" in peering_creation_result:
            return peering_creation_result

        peering_connection_id = peering_creation_result.get('VpcPeeringConnection', {}).get('VpcPeeringConnectionId')
        if not peering_connection_id:
            return {"Error": "VPC Peering Connection ID not found in creation response"}

        tag_result = self._run_aws_command([
            "aws", "ec2", "create-tags",
            "--resources", peering_connection_id,
            "--tags", f"Key=Name,Value={peering_name}",
            "--region", self.region_name
        ])

        if "Error" in tag_result:
            return tag_result

        return {"VpcPeeringConnectionId": peering_connection_id, "PeeringConnectionName": peering_name,
                "TagResult": tag_result}

    def _accept_vpc_peering_connection(self, peering_connection_id: str) -> dict:
        cmd = [
            "aws", "ec2", "accept-vpc-peering-connection",
            "--vpc-peering-connection-id", peering_connection_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _delete_vpc_peering_connection(self, peering_connection_id: str) -> dict:
        cmd = [
            "aws", "ec2", "delete-vpc-peering-connection",
            "--vpc-peering-connection-id", peering_connection_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _describe_vpc_peering_connections(self, peering_connection_id: str = None, vpc_id: str = None,
                                          peer_vpc_id: str = None) -> dict:
        cmd = [
            "aws", "ec2", "describe-vpc-peering-connections",
            "--region", self.region_name
        ]

        filters = []

        if peering_connection_id:
            filters.extend(["--vpc-peering-connection-ids", peering_connection_id])

        if vpc_id:
            filters.extend(["--filters", f"Name=requester-vpc-info.vpc-id,Values={vpc_id}"])

        if peer_vpc_id:
            filters.extend(["--filters", f"Name=accepter-vpc-info.vpc-id,Values={peer_vpc_id}"])

        cmd.extend(filters)

        return self._run_aws_command(cmd)

    def _reject_vpc_peering_connection(self, peering_connection_id: str) -> dict:
        cmd = [
            "aws", "ec2", "reject-vpc-peering-connection",
            "--vpc-peering-connection-id", peering_connection_id,
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    def _describe_vpc_peering_connection_requests(self, vpc_id: str = None) -> dict:
        cmd = [
            "aws", "ec2", "describe-vpc-peering-connections",
            "--filters", f"Name=requester-vpc-info.vpc-id,Values={vpc_id}",
            "--region", self.region_name
        ]

        return self._run_aws_command(cmd)

    @property
    def region(self) -> str:
        if self.region_name:
            return self.region_name


from ._vpc import _vpc
from .errors import ErrorHandler
from .routing_table import RoutingTable

class Subnet(_vpc):
    def __init__(self, region: str, subnet_id: str = None, aws_access_key: str = None,
                 aws_access_secret_key: str = None, vpc_id: str = None,
                 cidr_block: str = None, availability_zone: str = None):
        self._subnet_id = subnet_id
        self._region = region
        self._aws_access_key = aws_access_key
        self._aws_access_secret_key = aws_access_secret_key
        self._vpc_id = vpc_id
        self._cidr_block = cidr_block
        self._availability_zone = availability_zone
        self._error_handler = ErrorHandler()

        super().__init__(region=region, aws_access_key=self._aws_access_key, aws_access_secret_key=self._aws_access_secret_key)
        del self._aws_access_key, self._aws_access_secret_key
        self._initialize_subnet()

    def _initialize_subnet(self):
        if self._subnet_id is not None:
            subnet = super()._describe_subnet(self._subnet_id)
            self._availability_zone = subnet.get('AvailabilityZone')
            self._availability_zone_id = subnet.get('AvailabilityZoneId')
            self._available_ip_address_count = subnet.get('AvailableIpAddressCount')
            self._cidr_block = subnet.get('CidrBlock')
            self._default_for_az = subnet.get('DefaultForAz')
            self._map_public_ip_on_launch = subnet.get('MapPublicIpOnLaunch')
            self._map_customer_owned_ip_on_launch = subnet.get('MapCustomerOwnedIpOnLaunch')
            self._state = subnet.get('State')
            self._subnet_id = subnet.get('SubnetId')
            self._vpc_id = subnet.get('VpcId')
            self._owner_id = subnet.get('OwnerId')
            self._assign_ipv6_address_on_creation = subnet.get('AssignIpv6AddressOnCreation')
            self._ipv6_cidr_block_association_set = subnet.get('Ipv6CidrBlockAssociationSet')
            self._subnet_arn = subnet.get('SubnetArn')
            self._enable_dns64 = subnet.get('EnableDns64')
            self._ipv6_native = subnet.get('Ipv6Native')
            self._private_dns_name_options_on_launch = subnet.get('PrivateDnsNameOptionsOnLaunch')
        elif self._vpc_id is not None and self._cidr_block is not None and self._availability_zone is not None:
            x = super()._create_subnet(self._vpc_id, self._cidr_block, self._availability_zone)
            if "Error" in x:
                self._error_handler.parse_and_raise(x)
            else:
                self._subnet_id = x["SubnetId"]
                self._cidr_block = x["CidrBlock"]
                self._availability_zone = x["Subnet"]["AvailabilityZone"]
                self._initialize_subnet()

    def create_subnet(self, vpc_id: str, cidr_block: str, availability_zone: str):
        x = super()._create_subnet(vpc_id, cidr_block, availability_zone)
        if "Error" in x:
            self._error_handler.parse_and_raise(x)
        else:
            return Subnet(
                region=self._region,
                subnet_id=x["SubnetId"],
                aws_access_key=self._aws_access_key,
                aws_access_secret_key=self._aws_access_secret_key,
            )

    def delete_subnet(self, subnet_id: str = None) -> dict:
        if subnet_id is not None:
            return super()._delete_subnet(subnet_id)
        else:
            result = super()._delete_subnet(self._subnet_id)
            self.__del__()
            return result

    def disassociate_cidr_block(self, association_id: str) -> dict:
        result = super()._disassociate_subnet_cidr_block(association_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def get_cidr_reservations(self) -> dict:
        result = super()._get_subnet_cidr_reservations(self._subnet_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def modify_subnet_attribute(self, attribute_name: str, attribute_value) -> dict:
        result = super()._modify_subnet_attribute(self._subnet_id, attribute_name, attribute_value)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def __del__(self):
        # This function is invoked when the instance is about to be destroyed.
        # It helps in cleaning up any resources if necessary.
        for attr in list(self.__dict__.keys()):
            delattr(self, attr)

    @property
    def availability_zone(self):
        return self._availability_zone

    @property
    def availability_zone_id(self):
        return self._availability_zone_id

    @property
    def available_ip_address_count(self):
        return self._available_ip_address_count

    @property
    def cidr_block(self):
        return self._cidr_block

    @property
    def default_for_az(self):
        return self._default_for_az

    @property
    def map_public_ip_on_launch(self):
        return self._map_public_ip_on_launch

    @property
    def map_customer_owned_ip_on_launch(self):
        return self._map_customer_owned_ip_on_launch

    @property
    def state(self):
        return self._state

    @property
    def subnet_id(self):
        return self._subnet_id

    @property
    def vpc_id(self):
        return self._vpc_id

    @property
    def owner_id(self):
        return self._owner_id

    @property
    def assign_ipv6_address_on_creation(self):
        return self._assign_ipv6_address_on_creation

    @property
    def ipv6_cidr_block_association_set(self):
        return self._ipv6_cidr_block_association_set

    @property
    def ipv6_native(self):
        return self._ipv6_native

    @property
    def private_dns_name_options_on_launch(self):
        return self._private_dns_name_options_on_launch



from ._vpc import _vpc
from .errors import ErrorHandler
from .subnet import Subnet
from .routing_table import RoutingTable
from .peering_connection import PeeringConnection
from .internet_gateway import InternetGateway
from .nat_gateway import NatGateway

class VPC(_vpc):
    def __init__(self, region: str, vpc_id: str = None, aws_access_key: str = None, aws_access_secret_key: str = None,
                 vpc_cidr_block: str = None):
        self._vpc_id = vpc_id
        self._region = region
        self._aws_access_key = aws_access_key
        self._aws_access_secret_key = aws_access_secret_key
        self._vpc_cidr_block = vpc_cidr_block
        self._error_handler = ErrorHandler()

        super().__init__(region=region, aws_access_key=self._aws_access_key,
                         aws_access_secret_key=self._aws_access_secret_key)
        del self._aws_access_key, self._aws_access_secret_key
        self._initialize_vpc()

    def _initialize_vpc(self):
        if self._vpc_id is not None:
            vpc_data = super()._describe_vpc(self._vpc_id)
            if "Error" in vpc_data:
                print(vpc_data["Error"])
                self._error_handler.parse_and_raise(vpc_data)
            vpc_info = vpc_data.get('Vpcs', [])[0]  # Get the first VPC entry from the result

            self._cidr_block = vpc_info.get('CidrBlock')
            self._cidr_block_association_set = vpc_info.get('CidrBlockAssociationSet', [])
            self._dhcp_options_id = vpc_info.get('DhcpOptionsId')
            self._instance_tenancy = vpc_info.get('InstanceTenancy')
            self._is_default = vpc_info.get('IsDefault')
            self._state = vpc_info.get('State')
            self._owner_id = vpc_info.get('OwnerId')
            self._tags = vpc_info.get('Tags', [])
        elif self._vpc_cidr_block is not None:
            vpc_data = super()._create_vpc(self._vpc_cidr_block)
            if "Error" in vpc_data:
                self._error_handler.parse_and_raise(vpc_data)
            self._vpc_id = vpc_data.get('Vpc', {}).get('VpcId')
            self._initialize_vpc()

    def create_vpc(self, cidr_block: str) -> dict:
        vpc = super()._create_vpc(cidr_block)
        if "Error" in vpc:
            self._error_handler.parse_and_raise(vpc)
        self._vpc_id = vpc.get('Vpc', {}).get('VpcId')
        return vpc

    def delete_vpc(self, vpc_id: str = None) -> dict:
        vpc_id = vpc_id or self._vpc_id
        result = super()._delete_vpc(vpc_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def associate_cidr_block(self, cidr_block: str) -> dict:
        result = super()._associate_vpc_cidr_block(self._vpc_id, cidr_block)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def disassociate_cidr_block(self, association_id: str) -> dict:
        result = super()._disassociate_vpc_cidr_block(association_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def describe_cidr_reservations(self) -> dict:
        result = super()._describe_vpc_cidr_reservations(self._vpc_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def modify_attribute(self, attribute_name: str, attribute_value) -> dict:
        result = super()._modify_vpc_attribute(self._vpc_id, attribute_name, attribute_value)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def describe_vpc(self, vpc_id: str = None) -> dict:
        vpc_id = vpc_id or self._vpc_id
        result = super()._describe_vpc(vpc_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def _get_subnets(self):
        """
        Fetches all subnets associated with this VPC and returns a list of initialized Subnet objects.
        """
        result = super()._describe_subnets(self._vpc_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)

        subnets = result.get('Subnets', [])
        self._subnets = []

        for subnet_info in subnets:
            subnet = Subnet(
                region=self._region,
                subnet_id=subnet_info.get('SubnetId'),
                aws_access_key=self._access_key,
                aws_access_secret_key=self._secret_key,
                vpc_id=self._vpc_id
            )
            self._subnets.append(subnet)

        return self._subnets

    def _get_routing_tables(self):
        """
        Fetches all routing tables associated with this VPC and returns a list of initialized RoutingTable objects.
        """
        result = super()._describe_route_tables(self._vpc_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)

        routing_tables = result.get('RouteTables', [])
        self._routing_tables = []
        for rt_info in routing_tables:
            routing_table = RoutingTable(
                region=self._region,
                route_table_id=rt_info.get('RouteTableId'),
                aws_access_key=self._aws_access_key,
                aws_access_secret_key=self._aws_access_secret_key,
                vpc_id=self._vpc_id
            )
            self._routing_tables.append(routing_table)

        return self._routing_tables

    @property
    def cidr_block(self):
        return self._cidr_block

    @property
    def cidr_block_association_set(self):
        return self._cidr_block_association_set

    @property
    def dhcp_options_id(self):
        return self._dhcp_options_id

    @property
    def instance_tenancy(self):
        return self._instance_tenancy

    @property
    def is_default(self):
        return self._is_default

    @property
    def state(self):
        return self._state

    @property
    def owner_id(self):
        return self._owner_id

    @property
    def tags(self):
        return self._tags

    @property
    def vpc_id(self):
        return self._vpc_id

    @property
    def subnets(self):
        return self._get_subnets()

    @property
    def routing_tables(self):
        return self._routing_tables



    def __del__(self):
        # Cleanup resources if needed
        for attr in list(self.__dict__.keys()):
            delattr(self, attr)


if __name__ == '__main__':
    vpc = VPC('us-west-2', vpc_id="vpc-017f9600d16474436")
    subnets = vpc.subnets



from ._vpc import _vpc
from .errors import ErrorHandler

class RoutingTable(_vpc):
    def __init__(self, region: str, route_table_id: str = None, aws_access_key: str = None,
                 aws_access_secret_key: str = None, aws_session_token: str = None):
        self._route_table_id = route_table_id
        self._vpc_id = None
        self._region = region
        self._aws_access_key = aws_access_key
        self._aws_access_secret_key = aws_access_secret_key
        self._aws_sts_token = aws_session_token
        self._error_handler = ErrorHandler()

        super().__init__(region=region, aws_access_key=self._aws_access_key,
                         aws_access_secret_key=self._aws_access_secret_key)
        del self._aws_sts_token, self._aws_access_key, self._aws_access_secret_key
        self._initialize_route_table()

    def _initialize_route_table(self):
        if self._route_table_id is not None:
            route_table = super()._describe_route_table(self._route_table_id)
            if "Error" in route_table:
                self._error_handler.parse_and_raise(route_table)
            self._vpc_id = route_table.get('RouteTables', [{}])[0].get('VpcId')
            self._routes = route_table.get('RouteTables', [{}])[0].get('Routes')
        elif self._vpc_id is not None:
            route_table = super()._create_route_table(self._vpc_id)
            if "Error" in route_table:
                self._error_handler.parse_and_raise(route_table)
            self._route_table_id = route_table.get('RouteTableId')
            self._initialize_route_table()

    def create_route_table(self, vpc_id: str) -> dict:
        route_table = super()._create_route_table(vpc_id)
        if "Error" in route_table:
            self._error_handler.parse_and_raise(route_table)
        self._route_table_id = route_table.get('RouteTableId')
        return route_table

    def delete_route_table(self, route_table_id: str = None) -> dict:
        route_table_id = route_table_id or self._route_table_id
        result = super()._delete_route_table(route_table_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def associate_route_table(self, subnet_id: str) -> dict:
        result = super()._associate_route_table(self._route_table_id, subnet_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def disassociate_route_table(self, association_id: str) -> dict:
        result = super()._disassociate_route_table(association_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def create_route(self, destination_cidr_block: str, gateway_id: str = None, nat_gateway_id: str = None) -> dict:
        result = super()._create_route(self._route_table_id, destination_cidr_block, gateway_id, nat_gateway_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def delete_route(self, destination_cidr_block: str) -> dict:
        result = super()._delete_route(self._route_table_id, destination_cidr_block)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)
        return result

    def describe_route_tables(self):
        """
        Fetches all route tables associated with the VPC of this subnet and returns a list of RoutingTable objects.
        """
        if not self._vpc_id:
            raise ValueError("VPC ID is not available. Cannot describe route tables.")

        # Fetch route tables associated with the VPC
        result = super()._describe_route_tables(self._vpc_id)
        if "Error" in result:
            self._error_handler.parse_and_raise(result)

        routing_tables_info = result.get('RouteTables', [])
        routing_tables = []

        for rt_info in routing_tables_info:
            routing_table = RoutingTable(
                region=self._region,
                route_table_id=rt_info.get('RouteTableId'),
                aws_access_key=self._aws_access_key,
                aws_access_secret_key=self._aws_access_secret_key,
            )
            routing_tables.append(routing_table)

        return routing_tables

    @property
    def route_table_id(self):
        return self._route_table_id

    @property
    def vpc_id(self):
        return self._vpc_id

    @property
    def routes(self):
        return self._routes

    def __del__(self):
        # Cleanup resources if needed
        for attr in list(self.__dict__.keys()):
            delattr(self, attr)



import re as _re


class ClientNotFoundError(Exception):
    def __init__(self):
        super().__init__(
            "It appears like the AWS CLI client is not installed! To fix please visit 'https://docs.aws.amazon.com/cli/latest/userguide/getting-started-install.html'")


class SubnetError(Exception):
    def __init__(self, operation, message, *args):
        self.operation = operation
        self.message = message
        super().__init__(message, *args)


class SubnetCIDRConflicts(SubnetError):
    def __init__(self, operation, original_message, *args):
        if (cidr_match := _re.search(r"'(.*?)'", original_message)):
            cidr = cidr_match.group(1)
            formatted_message = f"The CIDR {cidr} conflicts with another subnet"
        else:
            formatted_message = "The CIDR conflicts with another subnet"
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidSubnet.Conflict"


class SubnetRangeError(SubnetError):
    def __init__(self, operation, original_message, *args):
        if (range_match := _re.search(r"'(.*?)'", original_message)):
            ip_range = range_match.group(1)
            formatted_message = f"The IP range {ip_range} is invalid or out of range"
        else:
            formatted_message = "The IP range is invalid or out of range"
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidSubnet.Range"


class SubnetIDNotFound(SubnetError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The specified subnet ID was not found."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidSubnet.ID.NotFound"


class SubnetZoneMismatch(SubnetError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The specified subnet is in the wrong availability zone."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidSubnet.ZoneMismatch"


class SubnetInUse(SubnetError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The subnet is currently in use and cannot be modified or deleted."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidSubnet.InUse"


class SubnetAssociationError(SubnetError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "There was an error associating the subnet with the specified resource."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidSubnet.Association"


class SubnetDependentServiceError(SubnetError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The subnet is being used by a dependent service and cannot be modified or deleted."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidSubnet.DependentService"


class SubnetAttachmentError(SubnetError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "There was an error attaching the subnet to the specified resource."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidSubnet.Attachment"


class VPCError(Exception):
    def __init__(self, operation, message, *args):
        self.operation = operation
        self.message = message
        super().__init__(message, *args)


class VPCCIDRConflicts(VPCError):
    def __init__(self, operation, original_message, *args):
        if (cidr_match := _re.search(r"'(.*?)'", original_message)):
            cidr = cidr_match.group(1)
            formatted_message = f"The CIDR {cidr} conflicts with another VPC"
        else:
            formatted_message = "The CIDR conflicts with another VPC"
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidVpc.Conflict"


class VPCIDNotFound(VPCError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The specified VPC ID was not found."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidVpc.ID.NotFound"


class VPCInUse(VPCError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The VPC is currently in use and cannot be modified or deleted."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidVpc.InUse"


class VPCDependentServiceError(VPCError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The VPC is being used by a dependent service and cannot be modified or deleted."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidVpc.DependentService"


class VPCAttachmentError(VPCError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "There was an error attaching the VPC to the specified resource."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidVpc.Attachment"


class RouteTableIDNotFound(VPCError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The specified route table ID was not found."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidRouteTableID.NotFound"


class InvalidRouteTableIDMalformed(VPCError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The specified route table ID is malformed."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidRouteTableID.Malformed"


class RouteNotFound(VPCError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The specified route was not found."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidRoute.NotFound"


class RouteAlreadyExists(VPCError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The specified route already exists."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "RouteAlreadyExists"


class InvalidRouteTableAssociationIDNotFound(VPCError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The specified route table association ID was not found."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidRouteTableAssociationID.NotFound"


class AccessDeniedError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "Access denied. You do not have the necessary permissions for this operation."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "AccessDenied"


class UnauthorizedOperationError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "You are not authorized to perform this operation."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "UnauthorizedOperation"


class AuthFailureError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "Authentication failed. Please check your AWS credentials."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "AuthFailure"


class RequestLimitExceededError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "Request limit exceeded. Please wait and try again later."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "RequestLimitExceeded"


class ThrottlingExceptionError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "Request was throttled. Please wait and try again later."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "ThrottlingException"


class ResourceNotFoundExceptionError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "The specified resource could not be found."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "ResourceNotFoundException"


class InvalidParameterValueError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "One or more parameters are invalid. Please check the input values."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidParameterValue"


class ServiceUnavailableError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "The service is currently unavailable. Please try again later."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "ServiceUnavailable"


class InternalFailureError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "An internal error occurred. Please try again later."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InternalFailure"


class ValidationExceptionError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "The input provided does not meet the required constraints."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "ValidationException"


class InvalidClientTokenIdError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "The security token included in the request is invalid."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidClientTokenId"


class OptInRequiredError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "You must opt in to use this service."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "OptInRequired"


class DependencyViolationError(Exception):
    def __init__(self, operation, message, *args):
        formatted_message = "The request failed because a resource is dependent on this resource."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "DependencyViolation"


class NatGatewayError(Exception):
    def __init__(self, operation, message, *args):
        self.operation = operation
        self.message = message
        super().__init__(message, *args)


class NatGatewayIDNotFound(NatGatewayError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The specified NAT Gateway ID was not found."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidNatGatewayID.NotFound"


class NatGatewayInUse(NatGatewayError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The NAT Gateway is currently in use and cannot be deleted."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "InvalidNatGateway.InUse"


class NatGatewayLimitExceeded(NatGatewayError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The maximum number of NAT Gateways has been reached."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "NatGatewayLimitExceeded"


class NatGatewayNotFound(NatGatewayError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The NAT Gateway was not found."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "NatGatewayNotFound"


class NatGatewayDependencyViolation(NatGatewayError):
    def __init__(self, operation, original_message, *args):
        formatted_message = "The NAT Gateway is being used by a dependent resource and cannot be deleted."
        super().__init__(operation, formatted_message, *args)
        self.error_code = "NatGatewayDependencyViolation"

class UnknownAWSClientError(Exception):
    def __init__(self, error_code):
        super().__init__(f"PLEASE READ: The AWS Client threw an unknown error code '{error_code}'! Please report this immediatly by submitting an issue on https://github.com/constallationlib/constallation-vpc and using the tag 'Unknown Client Code'")
        exit(0) # Halt immediately

class ErrorHandler:
    def parse_and_raise(self, error):
        error_message = error.get('Error', '')
        code_match = _re.search(r'\((.*?)\)', error_message)
        operation_match = _re.search(r'when calling the (.*?) operation', error_message)

        if code_match and operation_match:
            error_code = code_match.group(1)
            operation = operation_match.group(1)

            if error_code == "InvalidSubnet.Conflict":
                raise SubnetCIDRConflicts(operation, error_message)
            elif error_code == "InvalidSubnet.Range":
                raise SubnetRangeError(operation, error_message)
            elif error_code == "InvalidSubnet.ID.NotFound":
                raise SubnetIDNotFound(operation, error_message)
            elif error_code == "InvalidSubnet.ZoneMismatch":
                raise SubnetZoneMismatch(operation, error_message)
            elif error_code == "InvalidSubnet.InUse":
                raise SubnetInUse(operation, error_message)
            elif error_code == "InvalidSubnet.Association":
                raise SubnetAssociationError(operation, error_message)
            elif error_code == "InvalidSubnet.DependentService":
                raise SubnetDependentServiceError(operation, error_message)
            elif error_code == "InvalidSubnet.Attachment":
                raise SubnetAttachmentError(operation, error_message)
            elif error_code == "InvalidVpc.Conflict":
                raise VPCCIDRConflicts(operation, error_message)
            elif error_code == "InvalidVpc.ID.NotFound":
                raise VPCIDNotFound(operation, error_message)
            elif error_code == "InvalidVpc.InUse":
                raise VPCInUse(operation, error_message)
            elif error_code == "InvalidVpc.DependentService":
                raise VPCDependentServiceError(operation, error_message)
            elif error_code == "InvalidVpc.Attachment":
                raise VPCAttachmentError(operation, error_message)
            elif error_code == "InvalidRouteTableID.NotFound":
                raise RouteTableIDNotFound(operation, error_message)
            elif error_code == "InvalidRouteTableID.Malformed":
                raise InvalidRouteTableIDMalformed(operation, error_message)
            elif error_code == "InvalidRouteTableAssociationID.NotFound":
                raise InvalidRouteTableAssociationIDNotFound(operation, error_message)
            elif error_code == "InvalidRoute.NotFound":
                raise RouteNotFound(operation, error_message)
            elif error_code == "RouteAlreadyExists":
                raise RouteAlreadyExists(operation, error_message)
            elif error_code == "AccessDenied":
                raise AccessDeniedError(operation, error_message)
            elif error_code == "UnauthorizedOperation":
                raise UnauthorizedOperationError(operation, error_message)
            elif error_code == "AuthFailure":
                raise AuthFailureError(operation, error_message)
            elif error_code == "RequestLimitExceeded":
                raise RequestLimitExceededError(operation, error_message)
            elif error_code == "ThrottlingException":
                raise ThrottlingExceptionError(operation, error_message)
            elif error_code == "ResourceNotFoundException":
                raise ResourceNotFoundExceptionError(operation, error_message)
            elif error_code == "InvalidParameterValue":
                raise InvalidParameterValueError(operation, error_message)
            elif error_code == "ServiceUnavailable":
                raise ServiceUnavailableError(operation, error_message)
            elif error_code == "InternalFailure":
                raise InternalFailureError(operation, error_message)
            elif error_code == "ValidationException":
                raise ValidationExceptionError(operation, error_message)
            elif error_code == "InvalidClientTokenId":
                raise InvalidClientTokenIdError(operation, error_message)
            elif error_code == "OptInRequired":
                raise OptInRequiredError(operation, error_message)
            elif error_code == "DependencyViolation":
                raise DependencyViolationError(operation, error_message)
            elif error_code == "InvalidNatGatewayID.NotFound":
                raise NatGatewayIDNotFound(operation, error_message)
            elif error_code == "InvalidNatGateway.InUse":
                raise NatGatewayInUse(operation, error_message)
            elif error_code == "NatGatewayLimitExceeded":
                raise NatGatewayLimitExceeded(operation, error_message)
            elif error_code == "NatGatewayNotFound":
                raise NatGatewayNotFound(operation, error_message)
            elif error_code == "NatGatewayDependencyViolation":
                raise NatGatewayDependencyViolation(operation, error_message)
            else:
                raise UnknownAWSClientError(error_code)
        else:
            raise Exception("Unknown error occurred without specific details")
